filename = "${title} - ${new Date().format('yyyy-MM-dd')}.xlsx".toString() // "filename" is a standard Java String, not a GString

if (!binding.variables.containsKey('fieldHelper')) {
	throw new Exception("The script \"service-management-report.groovy\" requires the prior execution of \"field-helper-tool.groovy\".\nUse this expression after the last column of the topmost row: <mt:execute script=\"field-helper-tool.groovy\"/><mt:execute script=\"service-management-report.groovy\"/>")
}

serviceManagementReport = new ServiceManagementReportTool(fieldHelper: fieldHelper)

class ServiceManagementReportTool {
	def fieldHelper

	// - general --------------------------------------------------------------

	def getReporter(issue) {
		issue.reporter?.displayName ?: '(Anonymous)'
	}

	def getAssignee(issue) {
		issue.assignee?.displayName ?: '(Unassigned)'
	}

	def getRequestType(issue) {
		fieldHelper.getFieldValueByName(issue, 'Request Type') ?: '(No request type)'
	}

	def getRequestParticipants(issue) {
		fieldHelper.getFieldValueByName(issue, 'Request participants') ?: '(No participant)'
	}

	def getApprovers(issue) {
		fieldHelper.getFieldValueByName(issue, 'Approvers') ?: '(No approver)'
	}

	def getApproverGroups(issue) {
		fieldHelper.getFieldValueByName(issue, 'Approver groups') ?: '(No approver group)'
	}

	def getOrganizations(issue) {
		fieldHelper.getFieldValueByName(issue, 'Organizations') ?: '(No organization)'
	}

	def getUrgency(issue) {
		fieldHelper.getFieldValueByName(issue, 'Urgency') ?: '(No urgency)'
	}

	def getImpact(issue) {
		fieldHelper.getFieldValueByName(issue, 'Impact') ?: '(No impact)'
	}

	def getRequestLanguage(issue) {
		fieldHelper.getFieldValueByName(issue, 'Request language') ?: '(No language)'
	}

	// - used by "change-status-report.xlsx" ----------------------------------

	def getChangeType(issue) {
		fieldHelper.getFieldValueByName(issue, 'Change type') ?: '(No change type)'
	}

	def getChangeRisk(issue) {
		fieldHelper.getFieldValueByName(issue, 'Change risk') ?: '(No change risk)'
	}

	def getChangeReason(issue) {
		fieldHelper.getFieldValueByName(issue, 'Change reason') ?: '(No change reason)'
	}

	def getPlannedStart(issue) {
		fieldHelper.getFieldValueByName(issue, 'Planned start') ?: '(No planned start)'
	}

	def getPlannedEnd(issue) {
		fieldHelper.getFieldValueByName(issue, 'Planned end') ?: '(No planned end)'
	}

	def getActualStart(issue) {
		fieldHelper.getFieldValueByName(issue, 'Actual start') ?: '(No actual start)'
	}

	def getActualEnd(issue) {
		fieldHelper.getFieldValueByName(issue, 'Actual end') ?: '(No actual end)'
	}

	// - used by "customer-satisfaction-report.xlsx" --------------------------

	/**
	 * Returns <code>null</code> instead of "(No satisfaction rating)" when there is no score set,
	 * to avoid "#DIV/0!" when calculating the average in the pivot table.
	 */
	def getSatisfactionScore(issue) {
		fieldHelper.getFieldValueByName(issue, 'Satisfaction') ?: null
	}

	// - used by "incident-categorization-report.xlsx" ------------------------

	def getSeverity(issue) {
		fieldHelper.getFieldValueByName(issue, 'Severity') ?: '(No severity)'
	}

	def getInvestigationReason(issue) {
		fieldHelper.getFieldValueByName(issue, 'Investigation reason') ?: '(No investigation reason)'
	}

	def getSource(issue) {
		fieldHelper.getFieldValueByName(issue, 'Source') ?: '(No source)'
	}

	def getPendingReason(issue) {
		fieldHelper.getFieldValueByName(issue, 'Pending reason') ?: '(No pending reason)'
	}

	def getProductCategorization(issue) {
		fieldHelper.getFieldValueByName(issue, 'Product categorization') ?: '(No product categorization)'
	}

	def getOperationalCategorization(issue) {
		fieldHelper.getFieldValueByName(issue, 'Operational categorization') ?: '(No operational categorization)'
	}

	def getStartDate(issue) {
		fieldHelper.getFieldValueByName(issue, 'Start date') ?: '(No start date)'
	}

	// - used by "service-request-sla-report.xlsx" ----------------------------
	
	def getTimeToFirstResponse(issue) {
		fieldHelper.getFieldValueByName(issue, 'Time to first response') ?: '(No time to first response)'
	}

	def getTimeToResolution(issue) {
		fieldHelper.getFieldValueByName(issue, 'Time to resolution') ?: '(No time to resolution)'
	}

	def getTimeToCloseAfterResolution(issue) {
		fieldHelper.getFieldValueByName(issue, 'Time to close after resolution') ?: '(No time to close after resolution)'
	}

	def getTimeToReviewNormalChange(issue) {
		fieldHelper.getFieldValueByName(issue, 'Time to review normal change') ?: '(No time to review normal change)'
	}

	/**
	 * Note that if the SLA field value is null, then both this method and isSlaBreached() can return 0.
	 */
	def isSlaMet(issue, customFieldName) {
		def slaCustomFieldValue = fieldHelper.getFieldValueByName(issue, customFieldName)
		((slaCustomFieldValue == null) || (slaCustomFieldValue.toFloat() < 0)) ? 0 : 1
	}

	/**
	 * See {@link #isSlaMet(java.lang.Object, java.lang.Object)}
	 */
	def isSlaBreached(issue, customFieldName) {
		def slaCustomFieldValue = fieldHelper.getFieldValueByName(issue, customFieldName)
		((slaCustomFieldValue == null) || (slaCustomFieldValue.toFloat() >= 0)) ? 0 : 1
	}
}